@@ -72,7 +72,7 @@ See (gnus)Formatting Variables."
   "Whether server browsing should take place in the group buffer.
 If nil, a faster, but more primitive, buffer is used instead."
   :group 'gnus-server-visual
-  :type 'string)
+  :type 'boolean)
 
 ;;; Internal variables.
 
@@ -380,7 +380,11 @@ The following commands are available:
      (concat "(gnus-server-set-info \"" server "\" '"
 	     (prin1-to-string info) ")"))
     (let* ((server (nth 1 info))
-	   (entry (assoc server gnus-server-alist)))
+	   (entry (assoc server gnus-server-alist))
+	   (cached (assoc server gnus-server-method-cache)))
+      (if cached
+	  (setq gnus-server-method-cache
+		(delq cached gnus-server-method-cache)))
       (if entry (setcdr entry info)
 	(setq gnus-server-alist
 	      (nconc gnus-server-alist (list (cons server info))))))))
