@@ -375,7 +375,7 @@ marks file will be regenerated properly by Gnus.")
   (nnmail-check-syntax)
   (let (result)
     (when nnmail-cache-accepted-message-ids
-      (nnmail-cache-insert (nnmail-fetch-field "message-id")))
+      (nnmail-cache-insert (nnmail-fetch-field "message-id") group))
     (if (stringp group)
 	(and
 	 (nnmail-activate 'nnml)
@@ -747,8 +747,8 @@ marks file will be regenerated properly by Gnus.")
       (when (buffer-name (cdar nnml-nov-buffer-alist))
 	(set-buffer (cdar nnml-nov-buffer-alist))
 	(when (buffer-modified-p)
-	  (nnmail-write-region 1 (point-max) nnml-nov-buffer-file-name
-			       nil 'nomesg))
+	  (nnmail-write-region (point-min) (point-max)
+			       nnml-nov-buffer-file-name nil 'nomesg))
 	(set-buffer-modified-p nil)
 	(kill-buffer (current-buffer)))
       (setq nnml-nov-buffer-alist (cdr nnml-nov-buffer-alist)))))
@@ -837,7 +837,7 @@ marks file will be regenerated properly by Gnus.")
 	   (progn
 	     (re-search-forward "\n\r?\n" nil t)
 	     (setq chars (- (point-max) (point)))
-	     (max 1 (1- (point)))))
+	     (max (point-min) (1- (point)))))
 	  (unless (zerop (buffer-size))
 	    (goto-char (point-min))
 	    (setq headers (nnml-parse-head chars (caar files)))
@@ -849,7 +849,7 @@ marks file will be regenerated properly by Gnus.")
 	(setq files (cdr files)))
       (save-excursion
 	(set-buffer nov-buffer)
-	(nnmail-write-region 1 (point-max) nov nil 'nomesg)
+	(nnmail-write-region (point-min) (point-max) nov nil 'nomesg)
 	(kill-buffer (current-buffer))))))
 
 (defun nnml-nov-delete-article (group article)
